@@ -3,7 +3,7 @@
  * 
  * @fileOverview Implementation of the second revision of the JSON Schema specification draft.
  * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
- * @version 1.5
+ * @version 1.6
  * @see http://github.com/garycourt/JSV
  */
 
@@ -754,36 +754,29 @@
 		"initializer" : function (instance) {
 			var link, extension, extended;
 			
-			do {
-				//if there is a link to the full representation, replace instance
-				link = instance._schema.getLink("full", instance);
-				if (link && instance._uri !== link && instance._env._schemas[link]) {
-					instance = instance._env._schemas[link];
-					return instance;  //retrieved schemas are guaranteed to be initialized
-				}
-				
-				//if there is a link to a different schema, update instance
-				link = instance._schema.getLink("describedby", instance);
-				if (link && instance._schema._uri !== link && instance._env._schemas[link]) {
-					instance._schema = instance._env._schemas[link];
-					continue;  //start over
-				}
-				
-				//extend schema
-				extension = instance.getAttribute("extends");
-				if (JSV.isJSONSchema(extension)) {
-					extended = JSV.inherits(extension, instance, true);
-					
-					instance = instance._env.createSchema(extended, instance._schema, instance._uri);
-				}
-				
-				break;  //get out of the loop
-			} while (true);
-	
+			//if there is a link to a different schema, set reference
+			link = instance._schema.getLink("describedby", instance);
+			if (link && instance._schema._uri !== link) {
+				instance.setReference("describedby", link);
+			}
+			
 			//if instance has a URI link to itself, update it's own URI
 			link = instance._schema.getLink("self", instance);
-			if (link) {
-				instance._uri = link;
+			if (JSV.typeOf(link) === "string") {
+				instance._uri = JSV.formatURI(link);
+			}
+			
+			//if there is a link to the full representation, set reference
+			link = instance._schema.getLink("full", instance);
+			if (link && instance._uri !== link) {
+				instance.setReference("full", link);
+			}
+			
+			//extend schema
+			extension = instance.getAttribute("extends");
+			if (JSV.isJSONSchema(extension)) {
+				extended = JSV.inherits(extension, instance, true);
+				instance = instance._env.createSchema(extended, instance._schema, instance._uri);
 			}
 			
 			return instance;
@@ -987,11 +980,6 @@
 		}
 	}, HYPERSCHEMA, "http://json-schema.org/links#");
 	
-	//We need to reinitialize these 3 schemas as they all reference each other
-	SCHEMA = ENVIRONMENT.createSchema(SCHEMA.getValue(), HYPERSCHEMA, "http://json-schema.org/schema#");
-	HYPERSCHEMA = ENVIRONMENT.createSchema(HYPERSCHEMA.getValue(), HYPERSCHEMA, "http://json-schema.org/hyper-schema#");
-	LINKS = ENVIRONMENT.createSchema(LINKS.getValue(), HYPERSCHEMA, "http://json-schema.org/links#");
-	
 	JSV.registerEnvironment("json-schema-draft-02", ENVIRONMENT);
 	if (!JSV.getDefaultEnvironmentID() || JSV.getDefaultEnvironmentID() === "json-schema-draft-01") {
 		JSV.setDefaultEnvironmentID("json-schema-draft-02");
