@@ -3,7 +3,7 @@
  * 
  * @fileOverview Implementation of the third revision of the JSON Schema specification draft.
  * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
- * @version 1.3
+ * @version 1.4
  * @see http://github.com/garycourt/JSV
  */
 
@@ -41,7 +41,6 @@
 (function () {
 	var O = {},
 		JSV = require('./jsv').JSV,
-		InitializationError,
 		TYPE_VALIDATORS,
 		ENVIRONMENT,
 		SCHEMA_00_JSON,
@@ -69,20 +68,6 @@
 		HYPERSCHEMA_03,
 		LINKS_03;
 	
-	InitializationError = function InitializationError(instance, schema, attr, message, details) {
-		Error.call(this, message);
-		
-		this.uri = instance.getURI();
-		this.schemaUri = schema.getURI();
-		this.attribute = attr;
-		this.message = message;
-		this.description = message;  //IE
-		this.details = details;
-	}
-	InitializationError.prototype = new Error();
-	InitializationError.prototype.constructor = InitializationError;
-	InitializationError.prototype.name = "InitializationError";
-	
 	TYPE_VALIDATORS = {
 		"string" : function (instance, report) {
 			return instance.getType() === "string";
@@ -118,8 +103,9 @@
 	};
 	
 	ENVIRONMENT = new JSV.Environment();
+	ENVIRONMENT.setOption("validateReferences", true);
+	ENVIRONMENT.setOption("enforceReferences", false);
 	ENVIRONMENT.setOption("strict", false);
-	ENVIRONMENT.setOption("validateReferences", false);  //updated later
 	
 	//
 	// draft-00
@@ -909,31 +895,22 @@
 		"initializer" : function (instance) {
 			var link, extension, extended;
 			
-			//if there is a link to a different schema, update instance
+			//if there is a link to a different schema, set reference
 			link = instance._schema.getLink("describedby", instance);
 			if (link && instance._schema._uri !== link) {
-				if (instance._env._schemas[link]) {
-					instance._schema = instance._env._schemas[link];
-					initializer = instance._schema.getValueOfProperty("initializer");
-					if (typeof initializer === "function") {
-						return initializer(instance);  //this function will finish initialization
-					} else {
-						return instance;  //no further initialization
-					}
-				} else if (instance._env._options["validateReferences"]) {
-					throw new InitializationError(instance, instance._schema, "{link:describedby}", "Unknown schema reference", link);
-				}
+				instance.setReference("describedby", link);
 			}
 			
-			//if there is a link to the full representation, replace instance
+			//if instance has a URI link to itself, update it's own URI
+			link = instance._schema.getLink("self", instance);
+			if (JSV.typeOf(link) === "string") {
+				instance._uri = JSV.formatURI(link);
+			}
+			
+			//if there is a link to the full representation, set reference
 			link = instance._schema.getLink("full", instance);
 			if (link && instance._uri !== link) {
-				if (instance._env._schemas[link]) {
-					instance = instance._env._schemas[link];
-					return instance;  //retrieved schemas are guaranteed to be initialized
-				} else if (instance._env._options["validateReferences"]) {
-					throw new InitializationError(instance, instance._schema, "{link:full}", "Unknown schema reference", link);
-				}
+				instance.setReference("full", link);
 			}
 			
 			//extend schema
@@ -942,12 +919,6 @@
 				extended = JSV.inherits(extension, instance, true);
 				instance = instance._env.createSchema(extended, instance._schema, instance._uri);
 			}
-	
-			//if instance has a URI link to itself, update it's own URI
-			link = instance._schema.getLink("self", instance);
-			if (JSV.typeOf(link) === "string") {
-				instance._uri = JSV.formatURI(link);
-			}
 			
 			return instance;
 		}
@@ -1031,11 +1002,6 @@
 	
 	LINKS_00 = ENVIRONMENT.createSchema(LINKS_00_JSON, HYPERSCHEMA_00, "http://json-schema.org/draft-00/links#");
 	
-	//We need to reinitialize these 3 schemas as they all reference each other
-	SCHEMA_00 = ENVIRONMENT.createSchema(SCHEMA_00.getValue(), HYPERSCHEMA_00, "http://json-schema.org/draft-00/schema#");
-	HYPERSCHEMA_00 = ENVIRONMENT.createSchema(HYPERSCHEMA_00.getValue(), HYPERSCHEMA_00, "http://json-schema.org/draft-00/hyper-schema#");
-	LINKS_00 = ENVIRONMENT.createSchema(LINKS_00.getValue(), HYPERSCHEMA_00, "http://json-schema.org/draft-00/links#");
-	
 	//
 	// draft-01
 	//
@@ -1064,11 +1030,6 @@
 	
 	LINKS_01 = ENVIRONMENT.createSchema(LINKS_01_JSON, HYPERSCHEMA_01, "http://json-schema.org/draft-01/links#");
 	
-	//We need to reinitialize these 3 schemas as they all reference each other
-	SCHEMA_01 = ENVIRONMENT.createSchema(SCHEMA_01.getValue(), HYPERSCHEMA_01, "http://json-schema.org/draft-01/schema#");
-	HYPERSCHEMA_01 = ENVIRONMENT.createSchema(HYPERSCHEMA_01.getValue(), HYPERSCHEMA_01, "http://json-schema.org/draft-01/hyper-schema#");
-	LINKS_01 = ENVIRONMENT.createSchema(LINKS_01.getValue(), HYPERSCHEMA_01, "http://json-schema.org/draft-01/links#");
-	
 	//
 	// draft-02
 	//
@@ -1119,13 +1080,18 @@
 				},
 				
 				"validator" : function (instance, schema, self, report, parent, parentSchema, name) {
-					var divisor;
+					var divisor, value, digits;
 					if (instance.getType() === "number") {
 						divisor = schema.getAttribute("divisibleBy");
 						if (divisor === 0) {
 							report.addError(instance, schema, "divisibleBy", "Nothing is divisible by 0", divisor);
-						} else if (divisor !== 1 && ((instance.getValue() / divisor) % 1) !== 0) {
-							report.addError(instance, schema, "divisibleBy", "Number is not divisible by " + divisor, divisor);
+						} else if (divisor !== 1) {
+							value = instance.getValue();
+							digits = Math.max((value.toString().split(".")[1] || " ").length, (divisor.toString().split(".")[1] || " ").length);
+							digits = parseFloat(((value / divisor) % 1).toFixed(digits));  //cut out floating point errors
+							if (0 < digits && digits < 1) {
+								report.addError(instance, schema, "divisibleBy", "Number is not divisible by " + divisor, divisor);
+							}
 						}
 					}
 				}
@@ -1169,11 +1135,6 @@
 	
 	LINKS_02 = ENVIRONMENT.createSchema(LINKS_02_JSON, HYPERSCHEMA_02, "http://json-schema.org/draft-02/links#");
 	
-	//We need to reinitialize these 3 schemas as they all reference each other
-	SCHEMA_02 = ENVIRONMENT.createSchema(SCHEMA_02.getValue(), HYPERSCHEMA_02, "http://json-schema.org/draft-02/schema#");
-	HYPERSCHEMA_02 = ENVIRONMENT.createSchema(HYPERSCHEMA_02.getValue(), HYPERSCHEMA_02, "http://json-schema.org/draft-02/hyper-schema#");
-	LINKS_02 = ENVIRONMENT.createSchema(LINKS_02.getValue(), HYPERSCHEMA_02, "http://json-schema.org/draft-02/links#");
-	
 	//
 	// draft-03
 	//
@@ -1487,35 +1448,24 @@
 				refLink = instance.getValueOfProperty("$ref"),
 				idLink = instance.getValueOfProperty("id");
 			
-			//if there is a link to a different schema, update instance
+			//if there is a link to a different schema, set reference
 			if (schemaLink) {
 				link = instance.resolveURI(schemaLink);
-				if (link && instance._schema._uri !== link) {
-					if (instance._env._schemas[link]) {
-						instance._schema = instance._env._schemas[link];
-						initializer = instance._schema.getValueOfProperty("initializer");
-						if (typeof initializer === "function") {
-							return initializer(instance);  //this function will finish initialization
-						} else {
-							return instance;  //no further initialization
-						}
-					} else if (instance._env._options["validateReferences"]) {
-						throw new InitializationError(instance, instance._schema, "$schema", "Unknown schema reference", link);
-					}
+				instance.setReference("describedby", link);
+			}
+			
+			//if instance has a URI link to itself, update it's own URI
+			if (idLink) {
+				link = instance.resolveURI(idLink);
+				if (JSV.typeOf(link) === "string") {
+					instance._uri = JSV.formatURI(link);
 				}
 			}
 			
-			//if there is a link to the full representation, replace instance
+			//if there is a link to the full representation, set reference
 			if (refLink) {
 				link = instance.resolveURI(refLink);
-				if (link && instance._uri !== link) {
-					if (instance._env._schemas[link]) {
-						instance = instance._env._schemas[link];
-						return instance;  //retrieved schemas are guaranteed to be initialized
-					} else if (instance._env._options["validateReferences"]) {
-						throw new InitializationError(instance, instance._schema, "$ref", "Unknown schema reference", link);
-					}
-				}
+				instance.setReference("full", link);
 			}
 			
 			//extend schema
@@ -1524,14 +1474,6 @@
 				extended = JSV.inherits(extension, instance, true);
 				instance = instance._env.createSchema(extended, instance._schema, instance._uri);
 			}
-	
-			//if instance has a URI link to itself, update it's own URI
-			if (idLink) {
-				link = instance.resolveURI(idLink);
-				if (JSV.typeOf(link) === "string") {
-					instance._uri = JSV.formatURI(link);
-				}
-			}
 			
 			return instance;
 		}
@@ -1582,22 +1524,14 @@
 		}
 	});
 	
-	ENVIRONMENT.setOption("validateReferences", true);
 	ENVIRONMENT.setOption("defaultSchemaURI", "http://json-schema.org/draft-03/schema#");  //update later
 	
-	//prevent reference errors
-	ENVIRONMENT.createSchema({}, true, "http://json-schema.org/draft-03/schema#");
-	ENVIRONMENT.createSchema({}, true, "http://json-schema.org/draft-03/hyper-schema#");
-	ENVIRONMENT.createSchema({}, true, "http://json-schema.org/draft-03/links#");
-	
 	SCHEMA_03 = ENVIRONMENT.createSchema(SCHEMA_03_JSON, true, "http://json-schema.org/draft-03/schema#");
 	HYPERSCHEMA_03 = ENVIRONMENT.createSchema(JSV.inherits(SCHEMA_03, ENVIRONMENT.createSchema(HYPERSCHEMA_03_JSON, true, "http://json-schema.org/draft-03/hyper-schema#"), true), true, "http://json-schema.org/draft-03/hyper-schema#");
-	LINKS_03 = ENVIRONMENT.createSchema(LINKS_03_JSON, true, "http://json-schema.org/draft-03/links#");
 	
 	ENVIRONMENT.setOption("defaultSchemaURI", "http://json-schema.org/draft-03/hyper-schema#");
 	
-	//We need to reinitialize these schemas as they reference each other
-	HYPERSCHEMA_03 = ENVIRONMENT.createSchema(HYPERSCHEMA_03.getValue(), HYPERSCHEMA_03, "http://json-schema.org/draft-03/hyper-schema#");
+	LINKS_03 = ENVIRONMENT.createSchema(LINKS_03_JSON, true, "http://json-schema.org/draft-03/links#");
 	
 	ENVIRONMENT.setOption("latestJSONSchemaSchemaURI", "http://json-schema.org/draft-03/schema#");
 	ENVIRONMENT.setOption("latestJSONSchemaHyperSchemaURI", "http://json-schema.org/draft-03/hyper-schema#");
